<?php


/**
 * Provides the SKOSImporterTermParser class.
 */
class SKOSImporterTermParser extends SKOSImporterParser {

  protected $concepts;

  public $language_default;
  public $language_mapping;


  public function getId() {
    return $this->id;
  }

  /**
   * Override parent::configDefaults().
   */
  public function configDefaults() {
    return array(
      'additional_data' => array(),
      'import_type' => '',
      'vocabulary_name' => '',
      'multi_language' => 0,
      'languages' => array(),
    );
  }


  /**
   * TODO: Move the creation of taxonomies to the processor.
   */
  public function parse (SKOSImporterSource $source, SKOSImporterFetcherResult $fetcher_result) {
    $parser = ARC2::getRDFXMLParser();
    $parser->parse('', $fetcher_result->getRaw());
    $index = $parser->getSimpleIndex(0);

    $this->initLanguages();

    // search for all concept schemes
    $concept_schemes  = array();
    $top_concept      = array();
    $this->concepts   = array();
    if ($this->config['import_type'] == SKOSIMPORTER_IMPORT_INTO_MANY_VOCABURARIES) {
      foreach ($index as $uri => $item) {
        //$this->addLanguageToList($item);
        if (isset($item[SKOSIMPORTER_RDFNS_PREFIX . 'type']) && ($item[SKOSIMPORTER_RDFNS_PREFIX . 'type'][0]['value'] == SKOSIMPORTER_SKOS_PREFIX . 'ConceptScheme')) {
          $concept_schemes[$uri] = $item;
        }
        else {
          $this->concepts[$uri] = $item;
        }
      }
    }
    else {
      foreach ($index as $uri => $item) {
        //$this->addLanguageToList($item);
        if (isset($item[SKOSIMPORTER_RDFNS_PREFIX . 'type'])) {
          $types = array();
          foreach ($item[SKOSIMPORTER_RDFNS_PREFIX . 'type'] as $type) {
            $types[] = $type['value'];
          }
        }

        if (in_array(SKOSIMPORTER_SKOS_PREFIX . 'ConceptScheme', $types)) {
          // modify conceptSchemes to topConcepts
          $top_concepts[] = array('value' => $uri, 'type' => 'uri');
          $item[SKOSIMPORTER_SKOS_PREFIX . 'prefLabel']  = $this->getConceptSchemeName($item);
          $item[SKOSIMPORTER_SKOS_PREFIX . 'definition'] = $this->getConceptSchemeDescription($item);
          $item[SKOSIMPORTER_SKOS_PREFIX . 'narrower']   = $item[SKOSIMPORTER_SKOS_PREFIX . 'hasTopConcept'];
          $this->concepts[$uri] = $item;
        }
        elseif (in_array(SKOSIMPORTER_SKOS_PREFIX . 'Concept', $types)) {
          $this->concepts[$uri] = $item;
        }
      }
      // modify topConcepts to concepts
      $this->linkTopConceptsToConceptScheme($top_concepts);
      // create new conceptScheme
      $concept_schemes[0] = array(
        SKOSIMPORTER_DCTERMS_PREFIX . 'title'       => array(array('value' => $this->config['vocabulary_name'], 'lang' => $this->language_default)),
        SKOSIMPORTER_DCTERMS_PREFIX . 'description' => array(array('value' => 'Automatically generated by SKOS Importer', 'lang' => $this->language_default)),
        SKOSIMPORTER_SKOS_PREFIX . 'hasTopConcept'  => $top_concepts,
      );
    }

    if (empty($concept_schemes)) {
      // TODO: show error if no concept scheme is available
      return new SKOSImporterParserResult();
    }
    else {
      $this->createTaxonomies($concept_schemes);
    }

    $items = $this->getItems($concept_schemes);

    $parser_result = new SKOSImporterParserResult();
    $parser_result->title = t('SKOS Parser');
    $parser_result->items = $items;

    unset($parser);
    unset($index);
    unset($concept_schemes);
    unset($this->concepts);

    return $parser_result;
  }


  public function initLanguages () {
    $languages = $this->config['language_mapping'];
    $this->language_default = ((isset($this->config['language_mapping'][LANGUAGE_NONE])) ? $this->config['language_mapping'][LANGUAGE_NONE] : language_default('language'));
    
    $this->language_mapping = array();
    foreach ($this->config['language_mapping'] as $drupal_language => $pp_language) {
      $this->language_mapping[$pp_language] = $drupal_language;
    }
  }


  /**
   * TODO: compare with the languages set on the configuration site; when no
   * language is set then -> LANGUAGE_NONE
   */
  /*protected function addLanguageToList ($item) {
    if (isset($item[SKOSIMPORTER_SKOS_PREFIX . 'prefLabel'])) {
      $labels = $item[SKOSIMPORTER_SKOS_PREFIX . 'prefLabel'];
    } elseif (isset($item[SKOSIMPORTER_DCTERMS_PREFIX . 'title'])) {
      $labels = $item[SKOSIMPORTER_DCTERMS_PREFIX . 'title'];
    } elseif (isset($item[SKOSIMPORTER_RDFS_PREFIX . 'label'])) {
      $labels = $item[SKOSIMPORTER_RDFS_PREFIX . 'label'];
    } else {
      // No label --> error.
      $labels = array();
    }

    if (is_array($labels) && !empty($labels)) {
      $list = language_list('enabled');
      foreach ($labels as $label) {
        if (!in_array($label['lang'], $this->language_list)) {
          $this->language_list[] = $label['lang'];

          if (strpos($label['lang'], '-')) {
            list($pp_code, $pp_sub_code) = explode('-', $label['lang']);
          } else {
            $pp_code = $label['lang'];
          }
          foreach ($list[1] as $lang) {
            if (strpos($lang->language, '-')) {
              list($code, $sub_code) = explode('-', $lang->language);
            } else {
              $code = $lang->language;
            }
            if ($code == $pp_code) {
              $this->language_mapping[$label['lang']] = $lang->language;
            }
          }
        }
      }
    }
  }*/


  protected function linkTopConceptsToConceptScheme ($schemeURIs) {
    foreach ($schemeURIs as $schemeURI) {
      if (isset($this->concepts[$schemeURI['value']])) {
        $scheme = $this->concepts[$schemeURI['value']];
        foreach ($scheme[SKOSIMPORTER_SKOS_PREFIX . 'hasTopConcept'] as $conceptURI) {
          if (isset($this->concepts[$conceptURI['value']])) {
            $this->concepts[$conceptURI['value']][SKOSIMPORTER_SKOS_PREFIX . 'broader'][] = array('value' => $schemeURI['value'], 'type' => 'uri');
          }
        }
      }
    }
  }


  /**
   * Get an element identified by $element_key of the given item.
   * The element key corresponds to the values in the array returned by
   * SKOSImporterParser::getMappingSources().
   *
   * This method is invoked from SKOSImporterProcessor::map() when a concrete item is
   * processed.
   *
   * @ingroup mappingapi
   *
   * @param $batch
   *   SKOSImporterImportBatch object containing the sources to be mapped from.
   * @param $element_key
   *   The key identifying the element that should be retrieved from $source
   *
   * @return
   *   The source element from $item identified by $element_key.
   *
   * @see SKOSImporterProcessor::map()
   * @see SKOSImporterCSVParser::getSourceElement()
   */
  public function getSourceElement(SKOSImporterSource $source, SKOSImporterParserResult $result, $element_key) {
    $item = $result->currentItem();
    return isset($item[$element_key]) ? $item[$element_key] : SKOSIMPORTER_EMPTY_VALUE;
  }


  public function getMappingSources () {
    return array(
      'vid' => array(
        'name'        => t('Taxonomy ID'),
        'description' => t('The taxonomy ID in which the concept is')
      ),
      'uri' => array(
        'name'        => t('Concept URI'),
        'description' => t('The URI from the concept')
      ),
      'pref_label' => array(
        'name'        => t('prefLabel'),
        'description' => t('The prefLabel of the concept')
      ),
      'alt_labels' => array(
        'name'        => t('altLabel'),
        'description' => t('The altLabels of the concepts')
      ),
      'definition' => array(
        'name'        => t('Definition'),
        'description' => t('The definition of the concept')
      ),
      'broader' => array(
        'name'        => t('Broader'),
        'description' => t('The broader concepts of the concept')
      ),
    ) + parent::getMappingSources();
  }


  protected function createTaxonomies (&$concept_schemes) {
    $taxonomies = taxonomy_vocabulary_get_names();
    foreach ($concept_schemes as $uri => $scheme) {
      $concept_schemes[$uri]['new'] = TRUE;
      $name = $this->getConceptSchemeName($scheme, $this->language_default);
      // check if taxonomy already exists
      foreach ($taxonomies as $taxonomy) {
        if ($name == trim($taxonomy->name)) {
          $concept_schemes[$uri]['new'] = FALSE;
          break;
        }
      }
      // TODO: if taxonomy exists check for all the SKOS fields
      if ($concept_schemes[$uri]['new']) {
        $taxonomy = $this->createTaxonomy($scheme);
      } else {
        $taxonomy = taxonomy_vocabulary_load($taxonomy->vid);
      }
      if (isset($taxonomy->i18n_mode) && $taxonomy->i18n_mode & I18N_MODE_LOCALIZE) {
        $this->updateTaxonomyTranslation($taxonomy->vid, $scheme);
      }
      $concept_schemes[$uri]['vid'] = $taxonomy->vid;
      $concept_schemes[$uri]['machine_name'] = $taxonomy->machine_name;
    }
  }


  /**
   * TODO: Add more SKOS fields
   */
  protected function createTaxonomy ($scheme) {
    // create taxonomy
    $name = $this->getConceptSchemeName($scheme, $this->language_default);
    $description = $this->getConceptSchemeDescription($scheme, $this->language_default);
    if ($description != '') {
      $description = substr($description, 0, 128);
    }
    $machine_name = substr(preg_replace('/[^a-z0-9]+/', '_', strtolower($name)), 0, 255);

    $vocab = (object) array(
      'name'          => $name,
      'machine_name'  => $machine_name,
      'description'   => $description,
      'relations'     => TRUE,
      'hierarchy'     => 2,
    );
    taxonomy_vocabulary_save($vocab);

    // create uri field
    $field_name = 'field_uri';
    $field      = field_info_field($field_name);
    if (is_null($field)) {
      $field = array(
        'field_name'  => $field_name,
        'type'        => 'text',
        'label'       => t('URI')
      );
      field_create_field($field);
    }

    // attach uri field to taxonomy
    $instance = array(
      'field_name'  => $field_name,
      'entity_type' => 'taxonomy_term',
      'bundle'      => $machine_name,
      'label'       => t('URI'),
      'description' => t('The URI from the concept'),
      'required'    => TRUE,
      'widget'      => array(
        'type'      => 'text_textfield',
        'weight'    => 3
      )
    );
    field_create_instance($instance);

    // create alt label field
    $field_name = 'field_alt_labels';
    $field      = field_info_field($field_name);
    if (is_null($field)) {
      $field = array(
        'field_name'  => $field_name,
        'type'        => 'text',
        'label'       => t('Alt Labels'),
        'settings'    => array(
          'max_length' => 8191,
        ),
      );
      field_create_field($field);
    }

    // attach alt label field to taxonomy
    $instance = array(
      'field_name'  => $field_name,
      'entity_type' => 'taxonomy_term',
      'bundle'      => $machine_name,
      'label'       => t('Alt Labels'),
      'description' => t('Comma separated list of alt labels'),
      'required'    => FALSE,
      'widget'      => array(
        'type'      => 'text_textfield',
        'weight'    => 4
      )
    );
    field_create_instance($instance);

    if (defined('I18N_MODE_LOCALIZE')) {
      $vocab->i18n_mode = I18N_MODE_LOCALIZE;
      db_update('taxonomy_vocabulary')
        ->fields(array('i18n_mode' => $vocab->i18n_mode))
        ->condition('vid', $vocab->vid)
        ->execute();
    }

    drupal_set_message(t('Created vocabulary %vocab_name (#%vid).', array('%vocab_name' => $vocab->name, '%vid' => $vocab->vid)));

    return $vocab;
  }


  protected function updateTaxonomyTranslation ($vid, $scheme) {
    $names = $this->getConceptSchemeName($scheme);
    $descriptions = $this->getConceptSchemeDescription($scheme);

    $source_name = $names[$this->language_default];
    $source_description = $descriptions[$this->language_default];

    foreach ($names as $name) {
      $lang = $name['lang'];
      if ($lang != $this->language_default) {
        i18n_string_translation_update(array('taxonomy', 'vocabulary', $vid, 'name'), $name['value'], $lang, $source_name);
      }
    }
    foreach ($descriptions as $description) {
      $lang = $description['lang'];
      if ($lang != $this->language_default) {
        i18n_string_translation_update(array('taxonomy', 'vocabulary', $vid, 'description'), $description['value'], $lang, $source_description);
      }
    }
  }


  protected function getItems ($concept_schemes) {
    $items = array();
    foreach ($concept_schemes as $scheme) {
      if (isset($scheme[SKOSIMPORTER_SKOS_PREFIX . 'hasTopConcept'])) {
        $scheme_items = $this->getItemsFromConcepts($scheme[SKOSIMPORTER_SKOS_PREFIX . 'hasTopConcept'], $scheme['vid']);
        $items = array_merge($items, $scheme_items);
      }
    }

    return $items;
  }


  protected function getItemsFromConcepts ($conceptURIs, $vid) {
    if (is_null($conceptURIs) || empty($conceptURIs)) {
      return array();
    }
    $i        = 0;
    $items    = array();
    $narrower = array();
    foreach ($conceptURIs as $conceptURI) {
      if (isset($this->concepts[$conceptURI['value']])) {
        $concept = $this->concepts[$conceptURI['value']];
        $pref_label = $this->getValue($concept[SKOSIMPORTER_SKOS_PREFIX . 'prefLabel']);
        if (!empty($pref_label[$this->language_default])) {
          $items[$i]['uri']         = $conceptURI['value'];
          $items[$i]['pref_label']  = $pref_label;
          $items[$i]['vid']         = $vid;
          $items[$i]['language']    = $this->language_default;
          if (isset($concept[SKOSIMPORTER_SKOS_PREFIX . 'definition'])) {
            $items[$i]['definition']  = $this->getValue($concept[SKOSIMPORTER_SKOS_PREFIX . 'definition']);
          }
          if (isset($concept[SKOSIMPORTER_SKOS_PREFIX . 'altLabel'])) {
            $items[$i]['alt_labels']  =  $this->joinLiterals($concept[SKOSIMPORTER_SKOS_PREFIX . 'altLabel'], $this->language_default);
          }
          if (isset($concept[SKOSIMPORTER_SKOS_PREFIX . 'broader'])) {
            $items[$i]['broader']     = $this->getUris($concept[SKOSIMPORTER_SKOS_PREFIX . 'broader']);
          }
          $i++;
        }
        if (isset($concept[SKOSIMPORTER_SKOS_PREFIX . 'narrower'])) {
          $narrower[] = $concept[SKOSIMPORTER_SKOS_PREFIX . 'narrower'];
        }
      }
    }
    foreach ($narrower as $conceptURIs) {
      $children_items = $this->getItemsFromConcepts($conceptURIs, $vid);
      $items = array_merge($items, $children_items);
    }

    unset($narrower);

    return $items;
  }


  protected function getConceptSchemeName ($scheme, $lang=NULL) {
    $name = is_null($lang) ? array() : '';

    if (isset($scheme[SKOSIMPORTER_DCTERMS_PREFIX . 'title'])) {
      $name = $this->getValue($scheme[SKOSIMPORTER_DCTERMS_PREFIX . 'title'], $lang);
    } elseif (isset($scheme[SKOSIMPORTER_RDFS_PREFIX . 'label'])) {
      $name = $this->getValue($scheme[SKOSIMPORTER_RDFS_PREFIX . 'label'], $lang);
    }

    return $name;
  }


  protected function getConceptSchemeDescription ($scheme, $lang=NULL) {
    $description = is_null($lang) ? array() : '';

    if (isset($scheme[SKOSIMPORTER_DCTERMS_PREFIX . 'subject'])) {
      $description = $this->getValue($scheme[SKOSIMPORTER_DCTERMS_PREFIX . 'subject'], $lang);
    } elseif (isset($scheme[SKOSIMPORTER_DCTERMS_PREFIX . 'description'])) {
      $description = $this->getValue($scheme[SKOSIMPORTER_DCTERMS_PREFIX . 'description'], $lang);
    }

    return $description;
  }


  protected function getValue ($values, $lang=NULL) {
    $data = is_null($lang) ? array() : '';

    if (is_array($values) && !empty($values)) {
      foreach ($values as $value) {
        if (is_null($lang)) {
          // TODO: Use correct mapping (e.g. zh --> zh-hans) as parameter for the parser.
          if (isset($this->language_mapping[$value['lang']])) {
            $data[$value['lang']] = array('value' => trim($value['value']), 'lang' => $this->language_mapping[$value['lang']]);
          }
        } elseif ($value['lang'] == $lang) {
          $data = trim($value['value']);
          break;
        }
      }
    }

    return $data;
  }


  protected function joinLiterals ($literals, $lang) {
    $data = array();
    foreach ($literals as $literal) {
      if ($literal['lang'] == $lang) {
        $data[] = trim($literal['value']);
      }
    }

    return join(', ', $data);
  }

  protected function getUris ($resources) {
    $data = array();
    foreach ($resources as $resource) {
      $data[] = trim($resource['value']);
    }

    return $data;
  }
}
